@@ -1,7 +1,7 @@
 package SQL::Translator::Schema::Table;
 
 # ----------------------------------------------------------------------
-# $Id: Table.pm,v 1.30 2004/11/27 16:32:46 schiffbruechige Exp $
+# $Id: Table.pm,v 1.37 2007-10-24 10:55:44 schiffbruechige Exp $
 # ----------------------------------------------------------------------
 # Copyright (C) 2002-4 SQLFairy Authors
 #
@@ -51,8 +51,7 @@ use base 'SQL::Translator::Schema::Object';
 
 use vars qw( $VERSION $FIELD_ORDER );
 
-$VERSION = sprintf "%d.%02d", q$Revision: 1.30 $ =~ /(\d+)\.(\d+)/;
-
+$VERSION = sprintf "%d.%02d", q$Revision: 1.27 $ =~ /(\d+)\.(\d+)/;
 
 # Stringify to our name, being careful not to pass any args through so we don't
 # accidentally set it to undef. We also have to tweak bool so the object is
@@ -238,7 +237,9 @@ C<SQL::Translator::Schema::Index> object.
         $index = $index_class->new( \%args ) or return 
             $self->error( $index_class->error );
     }
-
+    foreach my $ex_index ($self->get_indices) {
+       return if ($ex_index->equals($index));
+    }
     push @{ $self->{'indices'} }, $index;
     return $index;
 }
@@ -490,6 +491,14 @@ Returns a field by the name provided.
 
     my $self       = shift;
     my $field_name = shift or return $self->error('No field name');
+    my $case_insensitive = shift;
+    if ( $case_insensitive ) {
+    	$field_name = uc($field_name);
+    	foreach my $field ( keys %{$self->{fields}} ) {
+    		return $self->{fields}{$field} if $field_name eq uc($field);
+    	}
+    	return $self->error(qq[Field "$field_name" does not exist]);
+    }
     return $self->error( qq[Field "$field_name" does not exist] ) unless
         exists $self->{'fields'}{ $field_name };
     return $self->{'fields'}{ $field_name };
@@ -844,7 +853,7 @@ an array or array reference.
     push @{ $self->{'options'} }, @$options;
 
     if ( ref $self->{'options'} ) {
-        return wantarray ? @{ $self->{'options'} || [] } : $self->{'options'};
+        return wantarray ? @{ $self->{'options'} || [] } : ($self->{'options'} || '');
     }
     else {
         return wantarray ? () : [];
@@ -901,6 +910,95 @@ avoid the overload magic of the Field objects returned by the get_fields method.
     }
 }
 
+# ----------------------------------------------------------------------
+sub equals {
+
+=pod
+
+=head2 equals
+
+Determines if this table is the same as another
+
+  my $isIdentical = $table1->equals( $table2 );
+
+=cut
+
+    my $self = shift;
+    my $other = shift;
+    my $case_insensitive = shift;
+    
+    return 0 unless $self->SUPER::equals($other);
+    return 0 unless $case_insensitive ? uc($self->name) eq uc($other->name) : $self->name eq $other->name;
+    return 0 unless $self->_compare_objects(scalar $self->options, scalar $other->options);
+    return 0 unless $self->_compare_objects(scalar $self->extra, scalar $other->extra);
+
+    # Fields
+    # Go through our fields
+    my %checkedFields;
+    foreach my $field ( $self->get_fields ) {
+    	my $otherField = $other->get_field($field->name, $case_insensitive);
+    	return 0 unless $field->equals($otherField, $case_insensitive);
+    	$checkedFields{$field->name} = 1;
+    }
+    # Go through the other table's fields
+    foreach my $otherField ( $other->get_fields ) {
+    	next if $checkedFields{$otherField->name};
+    	return 0;
+    }
+
+    # Constraints
+    # Go through our constraints
+    my %checkedConstraints;
+CONSTRAINT:
+    foreach my $constraint ( $self->get_constraints ) {
+    	foreach my $otherConstraint ( $other->get_constraints ) {
+    		if ( $constraint->equals($otherConstraint, $case_insensitive) ) {
+    			$checkedConstraints{$otherConstraint} = 1;
+    			next CONSTRAINT;
+    		}
+    	}
+    	return 0;
+    }
+    # Go through the other table's constraints
+CONSTRAINT2:
+    foreach my $otherConstraint ( $other->get_constraints ) {
+    	next if $checkedFields{$otherConstraint};
+    	foreach my $constraint ( $self->get_constraints ) {
+    		if ( $otherConstraint->equals($constraint, $case_insensitive) ) {
+    			next CONSTRAINT2;
+    		}
+    	}
+    	return 0;
+    }
+
+    # Indices
+    # Go through our indices
+    my %checkedIndices;
+INDEX:
+    foreach my $index ( $self->get_indices ) {
+    	foreach my $otherIndex ( $other->get_indices ) {
+    		if ( $index->equals($otherIndex, $case_insensitive) ) {
+    			$checkedIndices{$otherIndex} = 1;
+    			next INDEX;
+    		}
+    	}
+    	return 0;
+    }
+    # Go through the other table's indices
+INDEX2:
+    foreach my $otherIndex ( $other->get_indices ) {
+    	next if $checkedIndices{$otherIndex};
+    	foreach my $index ( $self->get_indices ) {
+    		if ( $otherIndex->equals($index, $case_insensitive) ) {
+    			next INDEX2;
+    		}
+    	}
+    	return 0;
+    }
+
+	return 1;
+}
+
 # ----------------------------------------------------------------------
 
 =head1 LOOKUP METHODS
